/**
 * includes go here.
 *
 * Copyright (C) 2003 International Business Machines, Inc.
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * 	need api in libsysfs to get one attribute of one particular device.
 *	also an api to get all attributes of one device.
 *
 */


#ifndef _SYSDIAG_H_
#define _SYSDIAG_H_

#include <errno.h>
#include <ctype.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <mntent.h>
#include "libsysfs.h"
#include "tdb.h"
#include "spinlock.h"
 
/*#include "ht.h"*/



#define LOG_FILE 	" /var/log/messages"
#define GREP_OP 	"grep -wi "


#define SYSDIAG_EVLOG_KEY 	"EVLOGVIEW"
#define SYSDIAG_TEST_KEY 	"TEST$"
#define SYSDIAG_INQ_KEY  	"INQ$"
#define SYSDIAG_STAT_KEY 	"STAT$"
#define SYSDIAG_CLASS_STR	"class"
#define SYSDIAG_BUS_STR		"bus"
#define SYSDIAG_BLOCK_STR	"block"


#define DIAG_SHOW_VALUE		0x00000001
#define DIAG_DIAG_INFO		0x00000002
#define DIAG_DEVICE       	0x00000004
#define DIAG_LIST         	0x00000008
#define DIAG_INQ          	0x00000010
#define DIAG_LOGS         	0x00000020
#define DIAG_STAT         	0x00000040
#define DIAG_TEST         	0x00000080
#define DIAG_ALL          	0x00000100
#define DIAG_DRIVER       	0x00000200
#define DIAG_PAUSE        	0x00000400
#define DIAG_BUS          	0x00000800
#define DIAG_TOPO         	0x00001000
#define DIAG_CLASS       	0x00002000
#define DIAG_DB_RELOAD		0x00004000
#define DIAG_SHOW_OLD_LOGS	0x00008000
#define DIAG_BLOCK		0x00010000

#define SYSDIAG_DB_DATA_SIZE 512

#define CMD_OPTIONS "B:b:c:diLlRr:psthxv?"

#define CONF_FILE "/usr/local/share/sysdiag.conf"

TDB_CONTEXT *sysdiag_db;

struct section {
	struct section *next;
	char *name;
	char *buf;
};

struct diags {
	char *name;
	char *inq;
	char *test;
	char *stat;
};

struct dlist *diag_list;
int DIAG_OPTIONS;
int DIAG_LEVEL; 

char DEVICE_NAME[SYSFS_NAME_LEN];
char DRIVER_NAME[SYSFS_NAME_LEN];
char CLASS_NAME[SYSFS_NAME_LEN];
char BLOCK_NAME[SYSFS_NAME_LEN];
char TOPO_NAME[SYSFS_NAME_LEN];
char BUS_NAME[SYSFS_NAME_LEN];
char SYSFS_MOUNT_PATH[SYSFS_PATH_MAX];
char sysdiag_db_file[SYSFS_PATH_MAX];

extern int list_main(void);
extern int parse_main();
extern char * get_scsi_quad(char *);
// extern void free_diag(struct diags *diag);
extern char * show_logs(char *, int flag);
//extern int show_sysfs_root(char *rootname);
extern int list_root(char *rootname);
extern int sysdiag_add_record(char *key, char *data);
extern int sysdiag_db_print_all();
extern char *sysdiag_db_get_str(char *data, int element);
extern char *sysdiag_get_record(char *key);
extern int sysdiag_db_find_element(char *data, char *device);
extern int sysdiag_db_num_of_elements(char *data);

#endif /* _SYSDIAG_H_ */
