/***
 * This file has the code for the getting the event log information for the
 * devices.
 * Here we parse the syslog and evlog conf file for getting the rules of
 * logging of device information. And then search those logs for the device
 * logs.
 *
 * Copyright (C) 2003 International Business Machines, Inc.
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "sysdiag.h"

struct lines {
	char *value;
	struct lines *next;
};

char *line = NULL;
char *crit_log_file = NULL, *kern_log_file = NULL, *emerg_log_file = NULL;
char info[] = "*.crit";
char kern[] = "kern.*";
char emerg[] = "*.emerg";

/**
 * now we have the all the log file path. we now have to build the correct
 * grep path.
 *
 */
char * build_path(char *path, char *dev_num, int flag)
{
	int kern_flag = 0, emerg_flag = 0, info_flag = 0;
	int size = 0;
	if (kern_log_file != NULL)
		kern_flag = 1;
	if (emerg_log_file != NULL)
		emerg_flag = 1;
	if (crit_log_file != NULL)
		info_flag = 1;

	if (kern_flag & emerg_flag)
		if (strcmp(kern_log_file, emerg_log_file) == 0)
			emerg_flag = 0;

	if (kern_flag & info_flag)
		if (strcmp(kern_log_file, crit_log_file) == 0)
			info_flag = 0;

	if (info_flag & emerg_flag)
		if (strcmp(crit_log_file, emerg_log_file) == 0)
			emerg_flag = 0;

	if (emerg_flag) {
		size += strlen(emerg_log_file) +1;
		if (flag == 1)
			size++;
	}
	
	if (kern_flag) { 
		size += strlen(kern_log_file) +1;
		if (flag == 1)
			size++;
	}
	if (info_flag) {
		size += strlen(crit_log_file) +1;
		if (flag == 1)
			size++;
	}
	if (size == 0) {
		size += strlen(LOG_FILE);
		if (flag == 1)
			size++;
	}


	size += strlen(GREP_OP) + strlen(dev_num);

	path = (char *)calloc(size+1, sizeof(char));
	strncpy(path, GREP_OP, strlen(GREP_OP));
	strncat(path, dev_num, strlen(dev_num));

	if (kern_flag) {
		strncat(path, " ", 1);
		strncat(path, kern_log_file, strlen(kern_log_file));
		if (flag == 1)
			strncat(path, "*", 1);
	}
	if (emerg_flag) {
		strncat(path, " ", 1);
		strncat(path, emerg_log_file, strlen(emerg_log_file));
		if (flag == 1)
			strncat(path, "*", 1);
	}
	if (info_flag) {
		strncat(path, " ", 1);
		strncat(path, crit_log_file, strlen(crit_log_file));
		if (flag == 1)
			strncat(path, "*", 1);
	}
	return (char *)path;

}



/**
 * this will take ptr to buff and return a pointer to a valid line.
 * also this will take pointer to end of line that it had pointed to
 *
 */
struct lines *get_line(char *buf)
{
	char *ptr1,*ptr,*ptr2;
	int size;
	struct lines *line = NULL, *line_head = NULL;

	ptr = buf;
	while ((ptr1 = strchr(ptr, '\n')) != NULL) {
		ptr2 = ptr;

		if ((ptr2 = strchr(ptr, '#')) == ptr) {
			ptr = ptr1 +1;
			continue;
		}
		if (ptr2 > ptr1) {
			size = ptr1 - ptr;
		}else if (ptr2 == NULL)
			size = ptr1 - ptr;
		else
			size = ptr2 - ptr;

		line_head = line;
		line = (struct lines *)calloc(1, sizeof(struct lines));
		line->value = (char *)calloc(size+1, sizeof(char));
		strncpy(line->value, ptr, size);

		line->next = line_head;
		ptr = ptr1 + 1;
	}
	if ((ptr1 = strchr(ptr,'\0')) != NULL) {
		line_head = line;
		size = ptr1 - ptr;

		line = (struct lines *)calloc(1, sizeof(struct lines));
		line->value = (char *)calloc(size+1, sizeof(char));
		strncpy(line->value, ptr, size);
		line->next = line_head;
	}
	line_head = line;
	return (struct lines *)line_head;
}


/**
 * we need to look for *.info messages, and kern.* and *.emerg
 * get the line where these are make sure that they are not commented.
 */

int parse_syslog_conf(void)
{

	char cnf_file[] = "/etc/syslog.conf";
	int fd, size;
	char *buf;
	char *line = NULL, *ptr1= NULL, *ptr2= NULL, *ptr = NULL;
	struct lines *line_head = NULL, *curr = NULL;

	struct stat stat_buf;

	stat(cnf_file, &stat_buf);
	size = stat_buf.st_size;
	buf = (char *)calloc(size, sizeof(char));

	fd = open(cnf_file, O_RDONLY);
	if (fd == -1) {
		fprintf(stderr, "error reading file %s \n", cnf_file);
		return -1;
	}

	if (read(fd, buf, size) == -1) {
		fprintf(stderr, "error reading file %s \n", cnf_file);
		return -1;
	}
	close(fd);
	ptr = buf;
	if (buf == NULL) {
		fprintf(stderr, "empty buffer passed\n");
		free(buf);
		return 1;
	}
	line_head = get_line(buf);
	curr = line_head;
	while (curr != NULL) {
		size = 0;
		line = curr->value;
		line_head = curr->next;
		if (strstr(line, info)!= NULL) {
			if ((ptr1 = strchr(line, '/')) != NULL) {
				if ((ptr2 = strstr(ptr1, " ")) == NULL) {
					if ((ptr2 = strchr(ptr1, '\n'))
								== NULL) {
						if ((ptr2 = strchr(ptr1, '\0'))
								== NULL)
							size = strlen(ptr1);
						else size = ptr2 - ptr1;
					} else size = ptr2 -ptr1;

				} else size = ptr2 -ptr1;

				crit_log_file = (char *)calloc(size+1,
								sizeof(char));
				strncpy(crit_log_file, ptr1, size);
			}
		}
		if (strstr(line, kern)!= NULL) {
			if ((ptr1 = strchr(line, '/')) != NULL) {
				if ((ptr2 = strstr(ptr1, " ")) == NULL) {
					if ((ptr2 = strchr(ptr1, '\n'))
								== NULL) {
						if ((ptr2 = strchr(ptr1, '\0'))
								== NULL)
							size = strlen(ptr1);
						else size = ptr2 - ptr1;
					} else size = ptr2 -ptr1;

				} else size = ptr2 -ptr1;

				kern_log_file = (char *)calloc(size+1,
								sizeof(char));
				strncpy(kern_log_file, ptr1, size);
			}
		}
		if (strstr(line, emerg)!= NULL) {
			if ((ptr1 = strchr(line, '/')) != NULL) {
				if ((ptr2 = strstr(ptr1, " ")) == NULL) {
					if ((ptr2 = strchr(ptr1, '\n'))
								== NULL) {
						if ((ptr2 = strchr(ptr1, '\0'))
								== NULL)
							size = strlen(ptr1);
						else size = ptr2 - ptr1;
					} else size = ptr2 -ptr1;

				} else size = ptr2 -ptr1;

				emerg_log_file = (char *)calloc(size+1,
								sizeof(char));
				strncpy(emerg_log_file, ptr1, size);
			}
		}
		free(curr->value);
		free(curr);
		curr = line_head;
	}
	return 0;
}
/**
 * show_logs(dev_num, flag):
 * input: char *dev_num -> device name to get logs for.
 * 	  int flag	-> to show old messages.
 * output: path -> grep path with the device name and the logs to look at.
 */
char * show_logs(char *dev_num, int flag)
{	
	char *path = NULL;
	int size;

	if (parse_syslog_conf() == -1)
		return NULL;
	if ((emerg_log_file == NULL) && (kern_log_file == NULL)
					&& (crit_log_file == NULL)) {
		size = strlen(GREP_OP) + strlen(LOG_FILE) + strlen(dev_num);
		if (flag == 1)
			size++;
		
		path = (char *)calloc(size, sizeof(char));
		strncpy(path, GREP_OP, strlen(GREP_OP));
		strncat(path, dev_num, strlen(dev_num));
		strncat(path, LOG_FILE, strlen(LOG_FILE));
		if (flag == 1)
			strncat(path, "*", 1);
		
	} else
		path = build_path(path, (char *)dev_num, flag);
	return (char *)path;

}
