/**
 * this will parse the conf file for adding or removing
 * diagnostic tools.
 *
 * Copyright (C) 2003 International Business Machines, Inc.
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *	General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License along
 *	with this program; if not, write to the Free Software Foundation, Inc.,
 *	675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * 	need api in libsysfs to get one attribute of one particular device.
 *	also an api to get all attributes of one device.
 *
 *
 *
 */

#include "sysdiag.h"


int check_diags(struct dlist *diag_list)
{
	struct diags *curr = NULL;
	struct dlist *list = NULL;
	char *cur = NULL;
	char *data = NULL;

	data = (char *)sysdiag_get_record(SYSDIAG_CLASS_STR);

	dlist_for_each_data(diag_list, curr, struct diags) {
		if (strstr(data, curr->name) != NULL) {
			if (sysdiag_db_find_element(data, curr->name) == 0) {
				if (strcmp(curr->name, "globals") != 0)
					dlist_delete(diag_list,0);
			}
		}
	}
		
}


static void free_diags(struct diags *diag)
{
		free(diag->name);
		free(diag->inq);
		free(diag->test);
		free(diag->stat);
		free(diag);
		return;
}

static void free_diag(void *diag)
{
	free_diags((struct diags *) diag);
}


/**
 * We have individual sections, we need to get the diag strings
 * from them now.
 *
 */

struct diags * parse_diags(char *buf, struct diags *diag)
{
	
	char *ptr1 = NULL, *ptr2 = NULL;
	int size;

	if ((ptr1 = strstr(buf, "STAT=")) != NULL) {
		ptr1 += 5;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->stat = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->stat, ptr1, size);
	}
	if ((ptr1 = strstr(buf,"INQ=")) != NULL) {
		ptr1 += 4;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->inq = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->inq, ptr1, size);
	} 
	if ((ptr1 = strstr(buf, "TEST=")) != NULL) {
		ptr1 += 5;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		diag->test = (char *)calloc(size+1, sizeof(char));
		strncpy(diag->test, ptr1, size);
	}
	
	
	return diag;
}


/**
 * this will put the evlogview command line setup into the
 * db.
 */

int diag_evlog(char *buf)
{
	char *ptr1 = NULL, *ptr2 = NULL;
	char *data = NULL, key[sizeof(SYSDIAG_EVLOG_KEY) +1];
	int size = 0;

	if (geteuid() != 0)
		return 0;
	
	if ((ptr1 = strstr(buf, "EVLOGVIEW=")) != NULL) {
		ptr1 += 10;
		if ((ptr2 = strchr(ptr1, '\n')) == NULL) {
			ptr2 = strchr(ptr1, '\0');
		}
		ptr2--;
		size = ptr2 - ptr1;
		size++;
		data = (char *)calloc(size+1, sizeof(char));
		strncpy(data, ptr1, size);
		memset(key, 0, strlen(SYSDIAG_EVLOG_KEY));
		strncpy(key, SYSDIAG_EVLOG_KEY, sizeof(SYSDIAG_EVLOG_KEY));
		sysdiag_add_record(key, data);
		free(data);
		return 0;
	}
	
	return -1;



}

int free_sec(struct section *sec)
{
	free(sec->buf);
	free(sec->name);
	free(sec);
	return 0;

}


int free_secs(struct section *sec)
{
	struct section *sec_next = NULL;

	for (; sec != NULL; sec = sec_next) {
		sec_next = sec->next;
		free_sec(sec);
	}
	return 0;

}


/**
 * this will add the diags to the db.
 */
int diag_add_sysdiag(struct dlist *list)
{
	struct diags *cur = NULL;
	char key[SYSFS_NAME_LEN], data[512];
	
	if (geteuid() != 0)
		return 0;
	
	dlist_for_each_data(list, cur, struct diags) {
		memset(key, 0, SYSFS_NAME_LEN);
		memset(data, 0, 512);
		strncpy(key, "diag#", 5);
		strncat(key, cur->name, strlen(cur->name));
		strncat(data, "TEST$", 5);
		if (cur->test != NULL) {
			strncat(data, cur->test, strlen(cur->test));
		}
		strncat(data, "#", 1);
		strncat(data, "INQ$", 5);
		if (cur->inq != NULL) {
			strncat(data, cur->inq, strlen(cur->inq));
		}
		strncat(data, "#", 1);
		strncat(data, "STAT$", 5);
		if (cur->stat != NULL) {
			strncat(data, cur->stat, strlen(cur->stat));
		}
		sysdiag_add_record(key, data);
	}
	return 0;

}

/**
 * make diag info from the database.
 */
struct dlist *get_db_diag_info()
{
	struct diags *diag_head = NULL;
	struct dlist *list = NULL;
	char diag_key[]="diag_key";
	char *data1 = NULL, *data2 = NULL, *diag1 = NULL;
	char *s = NULL, *r = NULL;
	int num1 = 0, num2 =2, i,j;
	char key[SYSFS_NAME_LEN];
	
	data1 = (char *)sysdiag_get_record(diag_key);
	if (data1 == NULL)
		return NULL;
	memset(key, 0, SYSFS_NAME_LEN);
	num1 = sysdiag_db_num_of_elements(data1);
	list = dlist_new_with_delete(sizeof(struct diags), free_diag);
	for (i = 1; i < num1 ; i++) {
		diag_head = calloc(1, sizeof(struct diags));
		diag1 = (char *)sysdiag_db_get_str(data1, i);
		strncpy(key, "diags#", 6);
		strncat(key, diag1, strlen(data2));
		data2 = sysdiag_get_record(key);
		num2 = sysdiag_db_num_of_elements(data2);
		if (num2 != 0) {
			diag_head->name = calloc(1, strlen(diag1));
			strncpy(diag_head->name, diag1, strlen(diag1));
		}
		
		for ( j = 1; j <= num2; j++) {
			s = (char *)sysdiag_db_get_str(data2, j);
			if ((r = strstr(s, SYSDIAG_TEST_KEY))!= NULL) {
				r += 5;
				if (strlen(r) > 1) {
					diag_head->test = calloc(1, strlen(r));
					strncpy(diag_head->test, r, strlen(r));
				} else 
					diag_head->test = NULL;
			}
			
			if ((r = strstr(s, SYSDIAG_INQ_KEY))!= NULL) {
				r += 4;
				if (strlen(r) > 1) {
					diag_head->inq = calloc(1, strlen(r));
					strncpy(diag_head->inq, r, strlen(r));
				} else 
					diag_head->inq = NULL;
			}
			if ((r = strstr(s, SYSDIAG_STAT_KEY))!= NULL) {
				r += 5;
				if (strlen(r) > 1) {
					diag_head->stat = calloc(1, strlen(r));
					strncpy(diag_head->stat, r, strlen(r));
				} else 
					diag_head->stat = NULL;
			}
		}
		dlist_unshift(list, diag_head);
	}
	return (struct dlist *)list;


}

/**
 * look for first section open '[' and the next section open
 * '[' between these lie conf for current class. If there is no
 * second '[' then we look for the end of file char '\0'.
 *
 * compare name of section with the classes that are there.
 * create link list of sections like.
 *
 * struct section {
 * 	struct section *next;
 * 	char *name;
 * 	char *buf;
 * }
 * 
 * parse the conf file into section and create a link list.
 * return the head of link list.
 * 
 * Parse each section, and fill the relavent variables for each
 * diagnostic type.
 * 
 */
int parse_file(char *buf)
{
	struct section *sec = NULL, *snext = NULL;
	struct dlist *list = NULL;
	struct diags *diag_head = NULL;
	char *name=NULL, *sbuf = NULL;
	char *opensec = NULL, *closesec = NULL;
	int size = 0;
	int last = 0;
	opensec = closesec = buf;
	
	while (opensec !=NULL ) {
		
		if ((opensec = strchr(closesec, '[')) == NULL) {
			break;
		}
		
		opensec++;
		if ((closesec = strchr(opensec, ']')) == NULL) {
			break;
		}
		size = 0;
		size = closesec - opensec;
		name = (char *)calloc(size+1, sizeof(char));
		strncpy(name, opensec, size);

		closesec = strchr(closesec, '\n');
		if (closesec == NULL) {
			break;
		}
		sec = malloc(sizeof(struct section));
		sec->buf = NULL;
		sec->name = name;
		closesec++;
		opensec = strchr(closesec, '[');
		if (opensec == NULL) {
			opensec = strchr(closesec, '\0');
			last=1;
		}
		size = 0;
		size = opensec - closesec;
		sbuf = (char *)calloc(size, sizeof(char));
		memcpy(sbuf, closesec, size);
		sec->buf = sbuf;
		sec->next = snext;
		snext = sec;
		closesec = opensec;
		if (last)
			break;	
	}
	
	list = dlist_new_with_delete(sizeof(struct diags), free_diag);
		
	for ( snext=sec; snext!=NULL; snext=snext->next) {
		diag_head = calloc(1,sizeof(struct diags));
		
		if (strstr(snext->buf, "EVLOGVIEW=") != NULL) {
			diag_evlog(snext->buf);
			continue;
		}
		
		if ((diag_head = (struct diags *)parse_diags(snext->buf,
						diag_head)) == NULL) {
			fprintf(stderr, "error parsing diags\n");
			free(diag_head);
			continue;
			
		}		
		size = strlen(snext->name);
		size++;
		diag_head->name = (char *)calloc(size, sizeof(char));
		strncpy(diag_head->name, snext->name, size);
		
		dlist_unshift(list, diag_head);

	}
	
	free_secs(sec);
	check_diags(list);
	diag_list = list;
	diag_add_sysdiag(list);
	return 0;

}

int get_db_file(char *fbuf)
{
	int fd,size;
	char *str =NULL, *ptr = NULL;
	struct stat buf;

	if ((str = strstr(fbuf, "DB_FILE=")) == NULL) {
		fprintf(stdout, "sysdiag database file location not specified\n"
				"please specify the location in sysdiag.conf file"
				" under section Globals in following format\n"
				"[globals]\n"
				"DB_FILE= <location of database file>\n"
				);
		return -1;
	}
	str += 8;
	ptr = strchr(str, '\n');
	if (ptr == NULL) {
		ptr = strchr(str, '\0');
	}
	size = ptr - str;
	size;
	if (size >= SYSFS_PATH_MAX) {
		fprintf(stdout, "database file path to long\n");
		return -1;
	}
	memset(sysdiag_db_file, 0, SYSFS_PATH_MAX);
	strncpy(sysdiag_db_file, str, size);
	return 0;
}


int parse_main()
{
	int fd,size;
	char *fbuf = NULL;
	struct stat buf;

	if (stat(CONF_FILE, &buf) == -1) {
		fprintf(stderr, "failed getting information about file %s\n", 
				CONF_FILE); 
		return -1;
	}
	fd = open(CONF_FILE, O_RDONLY);
	if (fd == -1) {
		fprintf(stderr, "failed opening file %s\n", CONF_FILE);
		return -1;
	}

	size = buf.st_size;
	fbuf = (char *)calloc(size,sizeof(char));

	if (read(fd,fbuf,size) == -1) {
		fprintf(stderr, "failed reading file %s\n", CONF_FILE);
		close(fd);
		free(fbuf);
		return -1;
	}
	close(fd);
	if ( get_db_file(fbuf) == -1) {
		fprintf(stdout, "error getting database file information "
				"from the sysdiag.conf file\n");
		free(fbuf);
		return -1;
		
	}
	if (sysdiag_db_main(0, NULL) != 0) {
		free(fbuf);
		return -1;
	}
	if (parse_file(fbuf) != 0)
		fprintf(stdout, "error parseing the sysdiag.conf file\n");
	
	free(fbuf);
	return 0;

}

/**
 * this will check if we have the sysdiag information in the sysdiag db or 
 * not. 
 * if diagnostic info is already not present in a structure. then we get it
 * from db, if it is not in db then we parse the sysdiag.conf file for the
 * diagnostics.
 */
struct dlist *sysdiag_get_diag_list()
{

	if (diag_list != NULL)
		return (struct dlist *)diag_list;

	if ((diag_list = get_db_diag_info()) != NULL)
		return (struct dlist *)diag_list;
	parse_main();
	return diag_list;
}

